function Gamma=amsre(P,A,C,R)
% This code is an axiliary matlab code to "msres.m" that solves for the forward
% solution for the Markov-Switching Rational Expectations (MSRE) models. 
% This code solves numerically the MSRE models without predetermined variables 
% that can be solved analytically by the method of undetermined coefficients.
% Written by Seonghoon Cho, September 9, 2009
% For additional information, see "Characterizing
% Markov-Switching Rational Expectations:Technical Guide".
% Definitions: x and z are n by 1 and m by 1 vectors of endogenous and 
%              exogenous variables, respectively. There are S regimes.
% =========================================================================
% Model :  x(t)=E[A(s(t),s(t+1))x(t+1)|I(t)]+C(s(t))*z(t)
%               z(t)=R*z(t-1)+e(t)
%               where s(t) has S regimes, 1,...,S.
%     Solution: x(t)=Gamma(s(t))*z(t)
%   [1] Input arguments
%       [1-1] Required Input
%           P   : S by S transition matrix where ij-th element, 
%               p_ij=Pr(s(t+1)=j|s(t)=i)
%           A   : S by S cell array such that A{i,j}=A(s(t)=i,s(t+1)=j).
%                (If A=A(s(t)), then A is S by 1 cell array such that
%                   A{i,1}=A(s(t)=i).). size(A{i,j})=n by n
%       [1-2] Optional Input :If one wants to specify an optional input 
%                   argument k, set all optional input arguments 1,2,..k-1 = [] 
%           C   : S by 1 cell array such that C{i,1}=C(s(t)=i).
%                   - Default: C=eye(n) so m=n; 
%           R   : m by m VAR coefficient matrix of z(t)
%                   - Default: R=zeros(m,m)
%                   - If R is zeros, set R=[]. 
%   [1-2] Output arguments: 
%       Gamma is S by 1 cell array such that Gamma{i,1}=Gamma(s(t)=i).

% [0] Identify S (number of Regime), n (rows of A) and m (columns of C)
    S=size(P,1); 
    n=size(A{1,1},1); m=size(C{1,1},2);
    if size(A,2)==1, for i=1:S, for j=1:S, AA{i,j}=A{i,1}; end, end, A=AA; end
        
    if nargin==2,       C=[]; m=n;              R=zeros(m,m); end
    if nargin==3,             m=size(C{1,1},2); R=zeros(m,m); end
    if nargin==4,             m=size(C{1,1},2);               end 
    
% [1] Make B and R cell arrays if any one of them is not a cell array. 
    if isempty(C),      for j=1:S, C{j,1}=eye(n);           end, end    

% Computing Gamma analytically    
    Psi=[];    
        for i=1:S, 
            Psirow=[];
            for j=1:S,
                Psirow=[Psirow kron(R',P(i,j)*A{i,j})];
            end, 
            Psi=[Psi;Psirow];
        end
        
    vvC=[]; for i=1:S, vvC=[vvC;C{i}(:)]; end
    vvGamma=inv(eye(S*n*m)-Psi)*vvC;

    vGamma=reshape(vvGamma,n*m,S);
    for i=1:S, Gamma{i,1}=reshape(vGamma(:,i),n,m); end
    
 