function ll=likelihood(C,Y,Macro);
% this function computes log-likelihood function 
% Y contains yields in columns (3m, 2y, 5y and 10y)
% Macro contains macro variables (output gap and inflation) in columns
global num_reg num_s2 num_s3 n T Tau dt EX
 
% define global variables 
[T n]=size(Y); % T is the number of observations, n is the number of yields
Tau=[0.25 2 5 10]'; %maturities in years 
dt=0.25; %discretization interval
 
% number of regimes
num_reg=8;
num_s1=2;
num_s2=2;
num_s3=2;
 
% unconditional mean of the state vector
EX=[0, mean(Macro(:,2)), mean(Y(:,1))];
 
% evaluate likelihood function
ll=feval(@llike,C,Y,Macro);
 
function ll=llike(C,Y,Macro)
% compute log-likelihood function 
global num_reg num_s2 num_s3 n T Tau dt EX
 
% get model parameters
% conditional mean
m0=0;
m1=0;
m2_1=0;
m2_2=C(4);
% mean reversion
k0=C(5);
k1=C(6);
k2=C(7);
k3=C(8);
k4=C(9);
k5=C(10);
k6_1=C(11);
k6_2=C(12);
k7_1=C(13);
k7_2=C(14);
k8_1=C(15);
k8_2=C(16);
% conditional volatility
sig1_1=C(17);
sig1_2=C(18);
sig2_1=C(19);
sig2_2=C(20);
sig3_1=C(21);
sig3_2=C(22);
% measurement error
sig2eps=C(23);
% risk premium 
lam1=C(24);
lam2=C(25);
lam3=C(26);
lam11=C(27);
lam12=C(28);
lam13=C(29);
lam21=C(30);
lam22=C(31);
lam23=C(32);
lam31=C(33);
lam32=C(34);
lam33=C(35);
% regime probabilities
q0=C(36);
q1=C(37);
p0=C(38);
p1=C(39);
r0=C(40);
r1=C(41);
 
% compute probability matrix
% systematic policy regimes
Qp=[q0    1-q0
    1-q1  q1  ];
% volatility regimes
Pp=[p0    1-p0
    1-p1  p1  ];
%monetary shock regimes 
Rp=[r0   1-r0
    1-r1 r1  ];
%full probability matrix
P=kron(Qp,kron(Pp,Rp));
 
%casting structural equations to a matrix form 
m0_1=[m0 m1 m2_1]';
m0_2=[m0 m1 m2_2]';
 
B0_1=[1     0 k2
      -k5   1 0 
      -k7_1 0 1];
  
B0_2=[1     0 k2
      -k5   1 0 
      -k7_2 0 1];
  
B1_1=[1-k1  0     0
       0   1-k4   0
       0    0   k8_1];
   
B1_2=[1-k1  0     0
       0   1-k4   0
       0    0   k8_2];
 
B2_1=[k0 k2   0
      0  k3   0
      0  k6_1 0];
B2_2=[k0 k2   0
      0  k3   0
      0  k6_2 0];
  
invB0_1=inv(B0_1);
invB0_2=inv(B0_2);
 
%iterative solution of thee structural model]
%define matrix F (see technical appendix)
F1_1=invB0_1*B1_1;
F1_2=invB0_2*B1_2;
F2_1=invB0_1*B2_1;
F2_2=invB0_2*B2_2;
%compute reduced-form parameters Phi(s)
[Phi_1,Phi_2,error]=iter_solution(F1_1,F1_2,F2_1,F2_2,q0,q1);
Phi=cell(2,1);
Phi{1}=Phi_1; Phi{2}=Phi_2;
 
%compute reduced-form parameters mu 
A0_1=eye(3,3)-F2_1*(Qp(1,1)*Phi_1+Qp(1,2)*Phi_2);
A0_2=eye(3,3)-F2_2*(Qp(2,1)*Phi_1+Qp(2,2)*Phi_2);
A0=[A0_1       zeros(3,3)
    zeros(3,3) A0_2      ];
 
F2=[Qp(1,1)*F2_1 Qp(1,2)*F2_1 
    Qp(2,1)*F2_2 Qp(2,2)*F2_2];
AF=inv(A0-F2);
AQ=[Qp'-eye(2,2)
   ones(1,2) ];
e0=zeros(3,1); e0(3,1)=1;
q=inv(AQ'*AQ)*AQ'*e0;
for i=1:2
    for j=1:2
        b(i,j)=Qp(i,j)*q(i)/q(j);
    end
end
Phib=[b(1,1)*Phi_1 b(2,1)*Phi_1
      b(1,2)*Phi_2 b(2,2)*Phi_2];
invB=[invB0_1 zeros(3,3)
      zeros(3,3) invB0_2];
Z=inv(eye(6,6)-Phib)*AF*invB;  
z=q(1)*Z(3,:)+q(2)*Z(6,:);
m0=-(k0-k1)*EX(1)+k2*(EX(3)-EX(2));
m1=-k5*EX(1)-(k3-k4)*EX(2);
m2_1=(EX(3)-(z(1)+z(4))*m0-(z(2)+z(5))*m1-z(6)*m2_2)/z(3);
m=[m0 m1 m2_1 m0 m1 m2_2]';
mu=AF*invB*m;
mu_1=mu(1:3,1);
mu_2=mu(4:6,1);
Mu=cell(2,1);
Mu{1}=mu_1; Mu{2}=mu_2; 
 
%compute reduced-form volatility parameters Sigma
AS_1=inv(A0_1)*invB0_1;
AS_2=inv(A0_2)*invB0_2;
Gam=cell(num_s2*num_s3,1);
Gam{1}=diag([sqrt(sig1_1) sqrt(sig2_1) sqrt(sig3_1)]);
Gam{2}=diag([sqrt(sig1_1) sqrt(sig2_1) sqrt(sig3_2)]);   
Gam{3}=diag([sqrt(sig1_2) sqrt(sig2_2) sqrt(sig3_1)]);  
Gam{4}=diag([sqrt(sig1_2) sqrt(sig2_2) sqrt(sig3_2)]);
Sig=cell(num_reg,1);
Sig{1}=AS_1*Gam{1};  Sig{2}=AS_1*Gam{2};  Sig{3}=AS_1*Gam{3};  Sig{4}=AS_1*Gam{4};  
Sig{5}=AS_2*Gam{1};  Sig{6}=AS_2*Gam{2};  Sig{7}=AS_2*Gam{3};  Sig{8}=AS_2*Gam{4}; 
 
%compute conditional variance
V=cell(num_reg,1);
for i=1:num_reg
 V{i}=dt*Sig{i}*Sig{i}';
end
        
 
% compute stationary probabilities
A=[P'-eye(num_reg,num_reg)
   ones(1,num_reg) ];
e0=zeros(num_reg+1,1); e0(num_reg+1,1)=1;
Qst=inv(A'*A)*A'*e0;
 
%compute factor loadings for bond pricing with quadratic approximation
L0=[lam1 lam2 lam3]'; % 
L1=[lam11 lam12 lam13
    lam21 lam22 lam23
    lam31 lam32 lam33];
 
[a,b,c,d,f]=bond_price(P,Mu,Phi,L0,L1,V);
 
% compute log-likelihood function
 
X=[Macro(:,1:2) Y(:,1)]; %state vector
Q=Qst; %initialize probability distribution at stationary probabilities
Qf(1,:)=Q';
sum0=0;
 
for t=2:T  
    for i=1:num_reg
        s1=fix((i-1)/(num_s2*num_s3))+1;
        e=X(t,:)'-Mu{s1}-Phi{s1}*X(t-1,:)';
        log_px(i,1)=-0.5*e'*inv(V{i})*e-0.5*log(det(V{i}))-1.5*log(2*pi); %state-vector part of the likelihood function
    end
 
    for j=1:num_reg
        for i=2:n
            moment1=a{j,i}+X(t,:)*b{j,i};
            moment2=c{j,i}+X(t,:)*d{j,i}+X(t,:)*f{j,i}*X(t,:)';
            Yimp(1,i-1)=100*(moment1+0.5*(moment1^2-moment2))/Tau(i); %model-implied yield with quadratic approximation
        end
        e=Y(t,2:end)-Yimp; %pricing error
        log_py(j,1)=-0.5*sum(e.^2)/sig2eps-0.5*(n-1)*log(2*pi*sig2eps); %measurement part of the likelihood function
    end
    
   for i=1:num_reg
        sum1=0;
        for j=1:num_reg
            sum1=sum1+P(j,i)*Q(j);
        end
        log_ps(i,1)=log_py(i,1)+log_px(i,1)+log(sum1);
    end
 
    max_ps=log_ps(1); mi=1;
    for i=2:num_reg
        if (log_ps(i)>max_ps)
            max_ps=log_ps(i); mi=i;
        end
    end
 
    sum1=0;
    for i=1:num_reg
        if (i~=mi)
            sum1=sum1+exp(log_ps(i)-max_ps);
        end
    end
 
    log_prob=max_ps+log(1+sum1);
    for i=1:num_reg
        Q(i,1)=exp(log_ps(i)-log_prob);
    end
    Qf(t,:)=Q';
    sum0=sum0+log_prob;
end
 
ll=-sum0/(T-1); %log-likelihood
 
 
function [A,B,C,D,F]=bond_price(P,Mu,Phi,L0,L1,V)
% compute factor loadings for bond pricing using quadratic approximation
global num_reg num_s2 num_s3 n T Tau  dt
 
t_eps=0.25*dt;
delta=[0 0 0.01*dt]';
 
A0=cell(num_reg,1); B0=cell(num_reg,1); C0=cell(num_reg,1); D0=cell(num_reg,1); F0=cell(num_reg,1);
A=cell(num_reg,n); B=cell(num_reg,n); C=cell(num_reg,n); D=cell(num_reg,n); F=cell(num_reg,n);
MuQ=cell(num_reg,1); PhiQ=cell(num_reg,1);
 
for i=1:num_reg
    s1=fix((i-1)/(num_s2*num_s3))+1;
    MuQ{i}=Mu{s1}-V{i}*L0;
    PhiQ{i}=Phi{s1}-V{i}*L1;
    A0{i}=0; B0{i}=zeros(3,1); C0{i}=0; D0{i}=zeros(3,1); F0{i}=zeros(3,3);
end
 
k=1;
for t=dt:dt:Tau(end)
   for i=1:num_reg
      sumA=0; sumB=zeros(3,1); sumC=0; sumD=zeros(3,1); sumF=zeros(3,3);
      for j=1:num_reg
        sumA=sumA+P(i,j)*(A0{j}+MuQ{j}'*B0{j});
        sumB=sumB+P(i,j)*(PhiQ{j}'*B0{j});
        sumC=sumC+P(i,j)*(C0{j}+MuQ{j}'*D0{j}+MuQ{j}'*F0{j}*MuQ{j}+trace(F0{j}*V{j}));
        sumD=sumD+P(i,j)*PhiQ{j}'*(D0{j}+2*F0{j}'*MuQ{j});
        sumF=sumF+P(i,j)*PhiQ{j}'*F0{j}*PhiQ{j};        
      end
      A{i,k}=sumA;
      B{i,k}=delta+sumB;
      C{i,k}=sumC;
      D{i,k}=2*A{i,k}*delta+sumD;
     %F{i,k}=-delta*delta'+delta*B{i,k}'+delta'*B{i,k}+sumF; — old code with a typo
      F{i,k}=-delta*delta'+delta*B{i,k}'+B{i,k}*delta'+sumF; %corrected on February 9, 2017; we thank David Leather for pointing this out 
   end
   for i=1:num_reg
      A0{i}=A{i,k};
      B0{i}=B{i,k};
      C0{i}=C{i,k};
      D0{i}=D{i,k};
      F0{i}=F{i,k};
   end
   if abs(t-Tau(k))<t_eps
       k=k+1;
   end
end
 
